#include <bits/stdc++.h>

//using namespace std;

template <typename T>
struct Treap {
	struct Node {
		int priority;
		T key;
		std::unique_ptr<Node> l, r;
		size_t size = 0;

		Node(T key): priority(rand()), key(key) {}

		// Helper
		void print() {
			if (this->l) {
				this->l->print();
				std::cout << "K: " << this->key << ", P: " << this->priority << ", S: " << this->size << "\n";
				std::cout << "    | left:  " << this->l->key << "\n";

				if (this->r) {
					std::cout << "    | right: " << this->r->key << "\n";
					this->r->print();
				}
			} else if (this->r) {
				std::cout << "K: " << this->key << ", P: " << this->priority << ", S: " << this->size << "\n";
				std::cout << "    | right: " << this->r->key << "\n";
				this->r->print();
			} else {
				std::cout << "K: " << this->key << ", P: " << this->priority << ", S: " << this->size << "\n";
			}
		}
	};

	std::unique_ptr<Node> root;

	void insert(const T& key) {
		auto node = std::make_unique<Node>(key);

		if (root == NULL) {
			root = std::move(node);
		} else {
			auto [l, r] = split(std::move(root), key);
			root = merge(merge(std::move(l), std::move(node)), std::move(r));
		}
	}

	void erase(const T& key) {
		if (root == NULL)
			return;

		std::unique_ptr<Node> l, r, eq;

		tie(l, r)  = split(std::move(root), key);
		tie(l, eq) = split(std::move(l), key - 1);

		eq = merge(std::move(eq->l), std::move(eq->r)); // Remove node with max prioirity
		root = merge(merge(std::move(l), std::move(eq)), std::move(r));
	}

	T at(size_t index) {
		Node* current = root.get();

		while (current != NULL) {
			size_t leftSize = (current->l != NULL ? current->l->size + 1 : 0);

			if (index < leftSize) {
				current = current->l.get();
			} else if (index > leftSize) {
				current = current->r.get();
				index -= leftSize + 1;
			} else {
				return current->key;
			}
		}

		throw std::out_of_range("");
	}

	size_t size() const {
		return (root == NULL ? 0 : root->size + 1);
	}

	int b();

private:

	static std::pair<std::unique_ptr<Node>, std::unique_ptr<Node>> split(std::unique_ptr<Node> current, const T& key) {
		std::unique_ptr<Node> l, r;

		if (current == NULL)
			return {std::move(l), std::move(r)};
		if (key < current->key) {
			r = std::move(current);
			tie(l, r->l) = split(std::move(r->l), key);
			r->size -= (l == NULL ? 0 : l->size + 1);
		} else {
			l = std::move(current);
			tie(l->r, r) = split(std::move(l->r), key);
			l->size -= (r == NULL ? 0 : r->size + 1);
		}

		return {std::move(l), std::move(r)};
	}

	static std::unique_ptr<Node> merge(std::unique_ptr<Node> l, std::unique_ptr<Node> r) {
		if (l == NULL || r == NULL)
			return (l != NULL) ? std::move(l) : std::move(r);

		if (l->priority > r->priority) {
			l->size += r->size + 1;
			l->r = merge(std::move(l->r), std::move(r));
			return l;
		} else {
			r->size += l->size + 1;
			r->l = merge(std::move(l), std::move(r->l));
			return r;
		}
	}

};
